/*****************************************************
 * 自動販売機
 * 使用マイコン　PIC16F1938
 * 2020/08/21 コインセレクターで動作するように変更
 * 2023/09/20 商品を選択できるように変更
 * 2023/10/10 とりあえず完成
 * 
******************************************************/
#include <xc.h>
#include <stdio.h>
//#include "command.h"

/*****コンフィグレーション設定*********/
// コンフィギュレーション１の設定
#pragma config FOSC = INTOSC    // 内部ｸﾛｯｸを使用する(INTOSC)
#pragma config WDTE = OFF       // ｳｵｯﾁﾄﾞｯｸﾞﾀｲﾏｰ無し(OFF)
#pragma config PWRTE = ON       // 電源ONから64ms後にﾌﾟﾛｸﾞﾗﾑを開始する(ON)
#pragma config MCLRE = ON      // 外部ﾘｾｯﾄ信号を使用
#pragma config CP = OFF         // ﾌﾟﾛｸﾞﾗﾑﾒﾓﾘｰを保護しない(OFF)
#pragma config CPD = OFF        // ﾃﾞｰﾀﾒﾓﾘｰを保護しない(OFF)
#pragma config BOREN = ON       // 電源電圧降下常時監視機能ON(ON)
#pragma config CLKOUTEN = OFF   // CLKOUTﾋﾟﾝをRA6ﾋﾟﾝで使用する(OFF)
#pragma config IESO = OFF       // 外部・内部ｸﾛｯｸの切替えでの起動はなし(OFF)
#pragma config FCMEN = OFF      // 外部ｸﾛｯｸ監視しない(FCMEN_OFF)

// コンフィギュレーション２の設定
#pragma config WRT = OFF        // Flashﾒﾓﾘｰを保護しない(OFF)
#pragma config PLLEN = OFF      // 動作クロックを32MHzでは動作させない(OFF)
#pragma config STVREN = ON      // スタックがオーバフローやアンダーフローしたらリセットをする(ON)
#pragma config BORV = HI        // 電源電圧降下常時監視電圧(2.5V)設定(HI)
#pragma config LVP = OFF        // 低電圧プログラミング機能使用しない(OFF)


#define     RY1     RA6         //セレクタ用電源
#define     RY2     RC0         
#define     RY3     RC1
#define     RY4     RC2
#define     RY5     RC3

#define     SW1     RA0
#define     SW2     RA1
#define     SW3     RA2
#define     SW4     RA3
#define     SW5     RA4
#define     SW6     RA5
#define     SW7     RB1

#define     ON      1               //
#define     OFF     0               //

/* 内部クロック　８ＭＨｚ */
#define  _XTAL_FREQ     8000000		//ディレイ用

unsigned	int coin_in;        //硬貨カウント用
unsigned	int swich_on;       //商品選択状態
unsigned	int BOX[5];     //ボックスの在庫管理
unsigned    int price_set;      //設定代金
unsigned    int price_on;       //投入硬貨が設定代金に

unsigned    int all_open;       //
unsigned    char JP1[3];        //
unsigned    char JP2[7];        //


/********　サブ関数　************/
void putch(unsigned char byte);

/********　メイン開始　************/
void main(void) 
{

    /********　変数定義　************/
	/*******初期設定***** */
    OSCCON = 0x72;			//内臓クロック８HMz指定(BPLL Off)
    ANSELA = 0x00;			//A/D停止
    ANSELB = 0x00;			//A/D停止

    TRISA  = 0b00111111;		//    
    TRISB  = 0b11111111;		//
    TRISC  = 0b10110000;		//

    WPUB =   0xff;              //ポートＢプルアップ
    nWPUEN = 0;                 //プルアップ有効

    /* USART初期設定*/
    TXSTA = 0x24;				//シリアル送信設定
    RCSTA = 0x90;				//シリアル受信設定
	BAUDCON=0x08;				//
	SPBRG = 207;				//ボーレート  SPBRG=103時19.2kbps
								//             SPBRG=207時9600bps
/****************************************************
 *  外部割り込み設定　
 * RB0　ポート　High→Low時割り込み発生 
 * 
****************************************************/
	INTF = 0;					//外部割り込みフラグクリア
	INTE = 1;					//外部割り込み許可
	INTEDG = 0;					//立下り（LOW）時割り込み
//	IOCIE = 0;					//状態割り込み無効
   	PEIE = 1;					//周辺割り込み許可
	GIE = 1;					//全体割り込み許可 
/************　PICが動作しているか確認用　********************/	
	printf("\r\nHELLO Ver1.6 \r\n");	//\r　復帰　\n　改行
  //ジャンパー設定の取り込み
    JP1[1] = !RC5;      
    JP1[2] = !RC4;
            
    JP2[1] = !RB7;
    JP2[2] = !RB6;            
    JP2[3] = !RB5;
    JP2[4] = !RB4;
    JP2[5] = !RB3;
    JP2[6] = !RB2;
    
    printf("JP1[1] %d \r\n",JP1[1]);
    printf("JP1[2] %d \r\n",JP1[2]);
    
    printf("JP2[1] %d \r\n",JP2[1]);
    printf("JP2[2] %d \r\n",JP2[2]);    
    printf("JP2[3] %d \r\n",JP2[3]);
    printf("JP2[4] %d \r\n",JP2[4]);    
    printf("JP2[5] %d \r\n",JP2[5]);
    printf("JP2[6] %d \r\n",JP2[6]);    
    
    price_set = 1+JP2[1]+JP2[2]*2+JP2[3]*4;
    
    printf("PRICE = %d \r\n",price_set);
    
    RY1 = 0;       
    RY2 = 0;
    RY3 = 0;
    RY4 = 0;
    RY5 = 0;
        
    swich_on = 0;
            
    __delay_ms(300);        //300ms ウエイト 
	while (1) {
 
    // ボックスをすべてオープンする    
        if (!SW5)
        {                  
        //ボックスすべてオープン処理    
            printf("ALL OPEN \r\n");    
            RY1 = 0;                //セレクター電源オフ
        
            __delay_ms(950);
            RY2 = 1;                //ボックス１オープン
            __delay_ms(950);       
            RY2 = 0;

            __delay_ms(950);        
            RY3 = 1;                //ボックス２オープン
            __delay_ms(950);    
            RY3 = 0;

            __delay_ms(950);        
            RY4 = 1;                //ボックス３オープン
            __delay_ms(950);
            RY4 = 0;       

            __delay_ms(950);         
            RY5 = 1;                //ボックス４オープン
            __delay_ms(950);
            RY5 = 0;       
         
            __delay_ms(950);
//            RY1 = 1;                //セレクター電源オン 
            while(!SW5)continue;      //   
            printf("SW5 END \r\n");    
        }
        
        //商品セット　ボックスのフラグリセット。
        if (!SW6){

            printf("SW6 on \r\n");
            BOX[1] = 1;
            BOX[2] = 1;
            BOX[3] = 1;
            BOX[4] = 1;
            RY1 = ON;      
            while(!SW6)continue;        //
            printf("SW6 off \r\n");
        }
           
        if(price_on){           //商品選状態か？
           price_on = 0;        //
           RY1 = 0;             //コインセレクター電源ＯＦＦ
           swich_on = 1;        //商品選択までループ
  
            while(swich_on){              
                if (SW1 == 0 & BOX[1] == 1){          //ＢＯＸ１選択
                    RY2 = ON;
                    __delay_ms(950);    //ソレノイドＯＮ（1秒以内）
                    RY2 = OFF;          
                    BOX[1] = 0;         //在庫無しフラグ
                    swich_on = 0;       //商品選択終了
                }    

                if (SW2 == 0 & BOX[2] == 1){
                    RY3 = ON;
                    __delay_ms(950);
                    RY3 = OFF;
                    BOX[2] = 0;
                    swich_on = 0;
                }

                if (SW3 == 0 & BOX[3] == 1){
                    RY4 = ON;
                    __delay_ms(950);
                    RY4 = OFF;
                    BOX[3] = 0;
                    swich_on = 0;
                }

                if (SW4 == 0 & BOX[4] == 1){
                    RY5 = ON;
                    __delay_ms(950);
                    RY5 = OFF;
                    BOX[4] = 0;   
                    swich_on = 0;
                }
            }           
           __delay_ms(3000);
           //すべてのボックスが開かれたかチェック
           all_open = BOX[1]+BOX[2]+BOX[3]+BOX[4];
           if (all_open != 0){          //すべてのＢＯＸが開かれていたら（完売）セレクタの電源を入れない
                RY1 = ON;               //在庫が有る場合はセレクター電源ＯＮ
           }
        }   
    }   
}

 /************************************************************
 * printf用
 ***********************************************************/
void putch(unsigned char byte){
		RCSTAbits.SPEN=1;
		TXREG=byte;
		while(!TXIF)continue;
		TXIF=0;
}
 /************************************************************
  割り込み　処理関数
  外部割り込み　RB0がLOWになった時割り込み
  コインセレクターからの信号
 ***********************************************************/
void interrupt isr(void)
{
    //外部割り込み処理　RB0がLOW(コイン投入）になった時割り込み発生
	INTF = 0;						//フラグクリアー
    coin_in++;
    if (price_set == coin_in)
    {    
        price_on = 1;
        coin_in = 0;
	}		
}